<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Service\UserService;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class UserController extends Controller
{
    /**
     * 意见反馈
     * */
    public function feedback(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'user_id' => 'required|int',
            'desc' => 'required|string',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $userId = $request->input('user_id');
        $desc = $request->input('desc');

        # 保存
        $errno = UserService::saveFeedback($userId, $desc);

        return self::returnValue([], $errno);
    }

    /**
     * 商品收藏 / 商品浏览
     * */
    public function goodsCollect(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'user_id' => 'required|int',
            'item_id' => 'required|int',
            'shop_type' => 'required|int',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $userId = (int)$request->input('user_id');
        $itemId = $request->input('item_id');
        $params['pid'] = $request->input('pid');                                // 推广位id
        $params['item_title'] = $request->input('item_title');                  // 宝贝标题
        $params['item_short_title'] = $request->input('item_short_title');      // 宝贝短标题
        $params['item_price'] = $request->input('item_price') * 100;                  // 在售价（单位：分）
        $params['item_end_price'] = $request->input('item_end_price') * 100;          // 券后价（单位：分）
        $params['shop_type'] = $request->input('shop_type');                    // 店铺类型。1天猫 2淘宝
        $params['coupon_money'] = $request->input('coupon_money') * 100;              // 优惠券金额（单位：分）
        $params['item_pic'] = $request->input('item_pic');                      // 宝贝主图
        $params['type'] = $request->input('type', 1);                           // 1商品收藏 2商品浏览
        $params['is_del'] = $request->input('is_del', 0);
        $params['enable'] = $request->input('enable', 1);
        # 商品收藏信息处理
        $errno = UserService::goodsInterest($userId, $itemId, 1, $params);

        return self::returnValue([], $errno);
    }

    /**
     * 我的收藏/浏览历史
     * */
    public function myInterest(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'user_id' => 'required|int',
            'type' => ['required', Rule::in([1, 2])],
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $userId = (int)$request->input('user_id');
        $type = (int)$request->input('type');
        $page = $request->input('page', 1);
        $pageSize = $request->input('page_size', 20);

        # 获取我的收藏列表
        list($list, $count) = UserService::getUserInterestList($userId, $type, $page, $pageSize);

        return self::returnPageValue($list, $count, $pageSize, $page);
    }

    /**
     * 移除收藏/浏览历史
     * */
    public function removeInterest(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'user_id' => 'required|int',
            'ids' => 'required|string',
            'type' => ['required', Rule::in([1, 2])],
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $userId = $request->input('user_id');
        $type = $request->input('type');
        $ids = $request->input('ids');

        # 保存
        $errno = UserService::removeInterest($userId, $ids, $type);

        return self::returnValue([], $errno);
    }

    /**
     * 用户搜索关键词历史
     * */
    public function searchHistory(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'user_id' => 'required|int',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $userId = $request->input('user_id');
        $data = UserService::getSearchHistory($userId);

        return self::returnValue($data);
    }

    /**
     * 删除搜索历史
     * */
    public function delSearchHistory(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'user_id' => 'required|int',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $userId = $request->input('user_id');
        $errno = UserService::delSearchHistory($userId);

        return self::returnValue([], $errno);
    }

    /**
     * 用户分享记录保存
     * */
    public function shareRecord(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'user_id' => 'required|int',
            'ids' => 'required|string',
            'type' => ['required', Rule::in([1, 2])],
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $userId = $request->input('user_id');
        $type = $request->input('type');
        $ids = $request->input('ids');

        # 保存
        $errno = 0;
        $shareId = UserService::saveShareRecord($userId, $ids, $type, $errno);

        if($errno)
            return self::returnValue([], $errno);

        return self::returnValue(['share_id' => $shareId]);
    }

    /**
     * 获取用户分享记录详情
     * */
    public function shareInfo(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'uid' => 'required|int',
            'share_id' => 'required|int',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $userId = $request->input('uid');
        $shareId = $request->input('share_id');
        $errno = 0;
        $data = UserService::getShareInfo($userId, $shareId, $errno);

        return self::returnValue($data, $errno);
    }

    /**
     * 配置用户头像/昵称
     * */
    public function setInfo(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'user_id' => 'required|int',
            'nickname' => 'required|max:20',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $userId = $request->input('user_id');
        $type = $request->input('type', 1);     // 1：链接  2：文件
        $avatarUrl = $type==1 ? $request->input('avatar_file') : $request->file('avatar_file');
        $nickname = $request->input('nickname');

        $errno = 0;
        $data = UserService::setUserInfo($userId, $type, $avatarUrl, $nickname, $errno);

        return self::returnValue([], $errno);
    }
}
